#if !defined(__CSIMAPI_H_INCLUDED__)
#define __CSIMAPI_H_INCLUDED__

typedef unsigned char BYTE;
typedef unsigned short WORD;
typedef unsigned long int DWORD;
typedef WORD CatFID;

typedef struct
{
  WORD recordLength;
  WORD numberOfRecords;
  BYTE lengthOfTrailer;
  BYTE trailer[];  
} CatEFStatus;

typedef struct
{
  BYTE numberOfDFs;
  BYTE numberofEFs;
  BYTE CHV1Status;
  BYTE unblockCHV1Status;
  BYTE CHV2Status;
  BYTE unblockCHV2Status;
  BYTE lengthOfTrailer;
  BYTE trailer[];
} CatDFStatus;

typedef struct
{
  DWORD totalFileSize;
  WORD fileID;
  BYTE fileDescriptorByte;
  BYTE fileType;  // 00=RFU, 01=MF, 02=DF, 04=EF
  BYTE fileLifeCycleStatus;
  union
  {
    CatEFStatus ef;
    CatDFStatus df;
  } u;
} CatFileStatus;

typedef enum
{
  DEVICE_KEPYAD         = 0x01,
  DEVICE_DISPLAY        = 0x02,
  DEVICE_EARPIECE       = 0x03,
  DEVICE_CARD_READER_0  = 0x10,
  DEVICE_CARD_READER_1  = 0x11,
  DEVICE_CARD_READER_2  = 0x12,
  DEVICE_CARD_READER_3  = 0x13,
  DEVICE_CARD_READER_4  = 0x14,
  DEVICE_CARD_READER_5  = 0x15,
  DEVICE_CARD_READER_6  = 0x16,
  DEVICE_CARD_READER_7  = 0x17,
  DEVICE_CHANNEL_1      = 0x21,
  DEVICE_CHANNEL_2      = 0x22,
  DEVICE_CHANNEL_3      = 0x23,
  DEVICE_CHANNEL_4      = 0x24, 
  DEVICE_CHANNEL_5      = 0x25,
  DEVICE_CHANNEL_6      = 0x26,
  DEVICE_CHANNEL_7      = 0x27,
  DEVICE_SIM            = 0x81,
  DEVICE_ME             = 0x82,
  DEVICE_NETWORK        = 0x83
} CatDevice;

typedef enum
{
  CARD_READER_STATUS     = 0x00, 
  CARD_READER_IDENTIFIER = 0x01
} CatGetReaderStatusOptions;

typedef enum
{
  REC_ACC_MODE_NEXT, 
  REC_ACC_MODE_PREVIOUS,  
  REC_ACC_MODE_ABSOLUTE_CURRENT
} CatRecordAccessMode;

typedef enum {
  BEGINNING_FORWARD,
  END_BACKWARD,
  NEXT_FORWARD,
  PREVIOUS_BACKWARD
} CatSearchMode;

typedef enum
{
  PRESENT_AS_DATA_VALUES_NO_HELP                = 0x01,
  PRESENT_AS_DATA_VALUES_WITH_HELP              = 0x81,
  PRESENT_AS_NAVIGATION_OPTIONS_NO_HELP         = 0x03,
  PRESENT_AS_NAVIGATION_OPTIONS_WITH_HELP       = 0x83,
  DEFAULT_STYLE_NO_HELP                         = 0x00,
  DEFAULT_STYLE_WITH_HELP                       = 0x80
} CatSelectItemOptions;

typedef enum
{
  CAT_COMMAND_SUCCESSFUL                                = 0x00,
  CAT_COMMAND_SUCCESSFUL_WITH_PARTIAL_COMPREHENSION     = 0x01,
  CAT_COMMAND_SUCCESSFUL_WITH_MISSING_INFORMATION       = 0x02,
  CAT_REFRESH_SUCCESSFUL_WITH_ADDITIONAL_EFS_READ       = 0x03,
  CAT_COMMAND_SUCCESSFUL_BUT_ICON_NOT_FOUND             = 0x04,
  CAT_COMMAND_SUCCESSFUL_BUT_MODIFIED_BY_CALL_CONTROL   = 0x05,
  CAT_COMMAND_SUCCESSFUL_BUT_LIMITED_SERVICE            = 0x06,
  CAT_COMMAND_SUCCESSFUL_WITH_MODIFICATION              = 0x07,
  CAT_ABORTED_BY_USER                                   = 0x10,
  CAT_BACKWARD                                          = 0x11,
  CAT_NO_RESPONSE                                       = 0x12,
  CAT_HELP_REQUIRED                                     = 0x13,
  CAT_USSD_ABORTED_BY_USER                              = 0x14,
  CAT_ME_UNABLE_TO_PROCESS_COMMAND                      = 0x20,
  CAT_NETWORK_UNABLE_TO_PROCESS_COMMAND                 = 0x21,
  CAT_USER_REJECTED_SETUP_CALL                          = 0x22,
  CAT_USER_CLEARED_BEFORE_RELEASE                       = 0x23,
  CAT_ACTION_CONTRADICT_TIMER_STATE                     = 0x24,
  CAT_TEMP_PROBLEM_IN_CALL_CONTROL                      = 0x25,
  CAT_LAUNCH_BROWSER_ERROR                              = 0x26,
  CAT_COMMAND_BEYOND_ME_CAPABILITIES                    = 0x30,
  CAT_COMMAND_TYPE_NOT_UNDERSTOOD                       = 0x31,
  CAT_COMMAND_DATA_NOT_UNDERSTOOD                       = 0x32,
  CAT_COMMAND_NUMBER_NOT_KNOWN                          = 0x33,
  CAT_SS_RETURN_ERROR                                   = 0x34,
  CAT_SMS_RP_ERROR                                      = 0x35,
  CAT_REQUIRED_VALUES_MISSING                           = 0x36,
  CAT_USSD_RETURN_ERROR                                 = 0x37,
  CAT_MULTIPLE_CARD_COMMAND_ERROR                       = 0x38,
  CAT_PERMANENT_PROBLEM_IN_SMS_OR_CALL_CONTROL          = 0x39,
  CAT_BEARER_INDEPENDENT_PROTOCOL_ERROR                 = 0x3A
} CatGeneralResult;

typedef enum
{
  // Command monitoring events
  EVENT_TERMINAL_PROFILE_COMMAND,
  EVENT_STATUS_COMMAND,
  EVENT_ENVELOPE_COMMAND,
  // Application lifecycle events start here
  EVENT_APPLICATION_LIFECYCLE_INSTALL = 0x20,
  // Framework fabricated events start here
  EVENT_UPDATE_EF_SMS = 0x40,
  EVENT_PROFILE_DOWNLOAD,
  EVENT_FORMATTED_SMS_PP_UPD,
  EVENT_UNFORMATTED_SMS_PP_UPD,
  EVENT_MENU_SELECTION,
  EVENT_FORMATTED_SMS_PP_ENV,
  EVENT_UNFORMATTED_SMS_PP_ENV,
  EVENT_FORMATTED_SMS_PP_CB,
  EVENT_MENU_SELECTION_HELP_REQUEST,
  EVENT_CALL_CONTROL_BY_SIM,
  EVENT_MO_SHORT_MESSAGE_CONTROL_BY_SIM,
  EVENT_TIMER_EXPIRATION,
  EVENT_DOWNLOAD_MT_CALL_EVENT,
  EVENT_DOWNLOAD_CALL_CONNECTED_EVENT,
  EVENT_DOWNLOAD_CALL_DISCONNECTED_EVENT,
  EVENT_DOWNLOAD_LOCATION_STATUS_EVENT,
  EVENT_DOWNLOAD_USER_ACTIVITY_EVENT,
  EVENT_DOWNLOAD_IDLE_SCREEN_AVAILABLE_EVENT,
  EVENT_DOWNLOAD_CARD_READER_STATUS_EVENT,
  EVENT_DOWNLOAD_LANGUAGE_SELECTION_EVENT,
  EVENT_DOWNLOAD_BROWSER_TERMINATION_EVENT,
  EVENT_DOWNLOAD_DATA_AVAILABLE_EVENT,
  EVENT_DOWNLOAD_CHANNEL_STATUS_EVENT,
  EVENT_UNRECOGNIZED_ENVELOPE,
  EVENT_TERMINAL_RESPONSE,
  EVENT_APPLICATION_INSTALL
}
CatFrameworkEventType;

typedef enum {
  SMS_PP_DOWNLOAD_TAG          = 0xD1,
  CELL_BROADCAST_TAG           = 0xD2,
  MENU_SELECTION_TAG           = 0xD3,
  CALL_CONTROL_TAG             = 0xD4,
  MO_SHORT_MESSAGE_CONTROL_TAG = 0xD5,
  EVENT_DOWNLOAD_TAG           = 0xD6,
  TIMER_EXPIRATION_TAG         = 0xD7
} CatEnvelopeTagType;

typedef enum {
  MT_CALL_EVENT                = 0x00,
  CALL_CONNECTED_EVENT         = 0x01,
  CALL_DISCONNECTED_EVENT      = 0x02,
  LOCATION_STATUS_EVENT        = 0x03,
  USER_ACTIVITY_EVENT          = 0x04,
  IDLE_SCREEN_AVAILABLE_EVENT  = 0x05,
  CARD_READER_STATUS_EVENT     = 0x06,
  LANGUAGE_SELECTION_EVENT     = 0x07,
  BROWSER_TERMINATION_EVENT    = 0x08,
  DATA_AVAILABLE_EVENT         = 0x09,
  CHANNEL_STATUS_EVENT         = 0x0A
} CatEventType;

typedef enum
{
  SHOW_WITHOUT_TEXT = 0x00,
  SHOW_WITH_TEXT = 0x01
}
CatIconOption;

typedef enum
{
  NORMAL_PRIORITY_AUTO_CLEAR = 0x00,
  NORMAL_PRIORITY_USER_CLEAR = 0x80,
  HIGH_PRIORITY_AUTO_CLEAR = 0x01,
  HIGH_PRIORITY_USER_CLEAR = 0x81
}
CatDisplayTextOptions;

typedef enum
{
  YES_NO_OPTION_NO_HELP = 0x04,
  YES_NO_OPTION_WITH_HELP = 0x84,
  DIGITS_ONLY_NO_HELP = 0x00,
  DIGITS_ONLY_WITH_HELP = 0x80,
  SMS_CHARACTER_NO_HELP = 0x01,
  SMS_CHARACTER_WITH_HELP = 0x81,
  UCS2_CHARACTER_NO_HELP = 0x03,
  UCS2_CHARACTER_WITH_HELP = 0x83
}
CatGetInKeyOptions;

typedef enum
{
  PACKED_DIGITS_ONLY_NO_HELP = 0x08,
  PACKED_DIGITS_ONLY_WITH_HELP = 0x88,
  PACKED_DIGITS_ONLY_NO_ECHO_NO_HELP = 0x0C,
  PACKED_DIGITS_ONLY_NO_ECHO_WITH_HELP = 0x8C,
  UNPACKED_DIGITS_ONLY_NO_HELP = 0x00,
  UNPACKED_DIGITS_ONLY_WITH_HELP = 0x80,
  UNPACKED_DIGITS_ONLY_NO_ECHO_NO_HELP = 0x04,
  UNPACKED_DIGITS_ONLY_NO_ECHO_WITH_HELP = 0x84,
  PACKED_SMS_ALPHABET_NO_HELP = 0x09,
  PACKED_SMS_ALPHABET_WITH_HELP = 0x89,
  PACKED_SMS_ALPHABET_NO_ECHO_NO_HELP = 0x0D,
  PACKED_SMS_ALPHABET_NO_ECHO_HELP = 0x8D,
  UNPACKED_SMS_ALPHABET_NO_HELP = 0x01,
  UNPACKED_SMS_ALPHABET_WITH_HELP = 0x81,
  UNPACKED_SMS_ALPHABET_NO_ECHO_NO_HELP = 0x05,
  UNPACKED_SMS_ALPHABET_NO_ECHO_WITH_HELP = 0x85,
  UCS2_ALPHABET_NO_HELP = 0x03,
  UCS2_ALPHABET_WITH_HELP = 0x83,
  UCS2_ALPHABET_NO_ECHO_NO_HELP = 0x07,
  UCS2_ALPHABET_NO_ECHO_WITH_HELP = 0x87
}
CatGetInputOptions;

typedef enum
{
  GSM_MINUTES = 0x00,
  GSM_SECONDS = 0x01,
  GSM_TENTHS_OF_SECONDS = 0x02
}
CatTimeUnit;

typedef enum
{
  DIAL_TONE = 0x01,
  CALLER_BUSY = 0x02,
  CONGESTION = 0x03,
  RADIO_PATH_ACKNOWLEDGE = 0x04,
  CALL_DROPPED = 0x05,
  SPECIAL_INFORMATION_OR_ERROR = 0x06,
  CALL_WAITING_TONE = 0x07,
  RINGING_TONE = 0x08,
  GENERAL_BEEP = 0x10,
  POSITIVE_ACKNOWLEDGE_TONE = 0x11,
  NEGATIVE_ACKNOWLEDGE_TONE = 0x12
}
CatTone;

typedef enum
{
  DCS_UNKNOWN =         0x00,
  DCS_SMS =             0x01,
  DCS_BINARY =          0x02,
  DCS_UCS2 =            0x03,
  DCS_SMS_UNPACKED =    0x04,
  DCS_SMS_UNICODE =     0x08,
  DCS_SMS_PACKED =      0x24
}
CatDCSValue;

typedef enum
{
  LOCAL_INFORMATION = 0x00,
  IMEI_OF_THE_PHONE = 0x01,
  NETWORK_MEASUREMENTS = 0x02,
  DATE_TIME_AND_TIME_ZONE = 0x03,
  LANGUAGE_SETTING = 0x04,
  TIMING_ADVANCE = 0x05
}
CatProvideLocalInformationOptions;

typedef struct
{
  BYTE mobileCountryNetworkCodes[3];
  BYTE LAC[2];
  BYTE cellID[2];
} CatLocationInformation;

typedef struct
{
  BYTE MEStatus;
  BYTE timingAdvance;
} CatTimingAdvance;

typedef enum
{
  TON_UNKNOWN_AND_NPI_UNKNOWN = 0x80,
  TON_INTERNATIONAL_AND_NPI_UNKNOWN = 0x90,
  TON_NATIONAL_AND_NPI_UNKNOWN = 0xA0,
  TON_NETWORK_AND_NPI_UNKNOWN = 0xB0,
  TON_SUBSCRIBER_AND_NPI_UNKNOWN = 0xC0,

  TON_UNKNOWN_AND_NPI_TELEPHONE = 0x81,
  TON_INTERNATIONAL_AND_NPI_TELEPHONE = 0x91,
  TON_NATIONAL_AND_NPI_TELEPHONE = 0xA1,
  TON_NETWORK_AND_NPI_TELEPHONE = 0xB1,
  TON_SUBSCRIBER_AND_NPI_TELEPHONE = 0xC1,

  TON_UNKNOWN_AND_NPI_DATA = 0x83,
  TON_INTERNATIONAL_AND_NPI_DATA = 0x93,
  TON_NATIONAL_AND_NPI_DATA = 0xA3,
  TON_NETWORK_AND_NPI_DATA = 0xB3,
  TON_SUBSCRIBER_AND_NPI_DATA = 0xC3,

  TON_UNKNOWN_AND_NPI_TELEX = 0x84,
  TON_INTERNATIONAL_AND_NPI_TELEX = 0x94,
  TON_NATIONAL_AND_NPI_TELEX = 0xA4,
  TON_NETWORK_AND_NPI_TELEX = 0xB4,
  TON_SUBSCRIBER_AND_NPI_TELEX = 0xC4,

  TON_UNKNOWN_AND_NPI_NATIONAL = 0x88,
  TON_INTERNATIONAL_AND_NPI_NATIONAL = 0x98,
  TON_NATIONAL_AND_NPI_NATIONAL = 0xA8,
  TON_NETWORK_AND_NPI_NATIONAL = 0xB8,
  TON_SUBSCRIBER_AND_NPI_NATIONAL = 0xC8,

  TON_UNKNOWN_AND_NPI_PRIVATE = 0x89,
  TON_INTERNATIONAL_AND_NPI_PRIVATE = 0x99,
  TON_NATIONAL_AND_NPI_PRIVATE = 0xA9,
  TON_NETWORK_AND_NPI_PRIVATE = 0xB9,
  TON_SUBSCRIBER_AND_NPI_PRIVATE = 0xC9,

  TON_UNKNOWN_AND_NPI_ERMES = 0x8A,
  TON_INTERNATIONAL_AND_NPI_ERMES = 0x9A,
  TON_NATIONAL_AND_NPI_ERMES = 0xAA,
  TON_NETWORK_AND_NPI_ERMES = 0xBA,
  TON_SUBSCRIBER_AND_NPI_ERMES = 0xCA
}
CatTypeOfNumberAndNumberingPlanIdentifier;


typedef enum
{
  PACKING_NOT_REQUIRED = 0x00,
  PACKING_BY_THE_PHONE_REQUIRED = 0x01
}
CatSendShortMessageOptions;


typedef enum
{
  REFRESH_SIM_INIT_AND_FULL_FILE_CHANGE_NOTIFICatION = 0x00,
  REFRESH_FULL_FILE_NOTIFICatION = 0x01,
  REFRESH_SIM_INIT_AND_FILE_CHANGE_NOTIFICatION = 0x02,
  REFRESH_SIM_INIT = 0x03,
  REFRESH_SIM_RESET = 0x04
}
CatRefreshOptions;


typedef enum
{
  LANGUAGE_NON_SPECIFIC_NOTIFICATION = 0x00,
  LANGUAGE_SPECIFIC_NOTIFICATION = 0x01
}
CatLanguageNotificationOptions;

typedef enum
{
  CALL_ONLY_IF_NOT_BUSY = 0x00,
  CALL_ONLY_IF_NOT_BUSY_WITH_REDIAL = 0x01,
  CALL_AND_PUT_CURRENT_CALL_ON_HOLD = 0x02,
  CALL_AND_PUT_CURRENT_CALL_ON_HOLD_WITH_REDIAL = 0x03,
  CALL_AND_DISCONNECT_CURRENT_CALL = 0x04,
  CALL_AND_DISCONNECT_CURRENT_CALL_WITH_REDIAL = 0x05
}
CatSetupCallOptions;

typedef enum
{
  LAUNCH_BROWSER_WITHOUT_MAKING_CONNECTION = 0x00,
  LAUNCH_BROWSER_MAKING_A_CONNECTION = 0x01,
  LAUNCH_USE_EXISTING_BROWSER = 0x02,
  LAUNCH_NEW_BROWSER_MAKING_A_CONNECTION = 0x03,
  LAUNCH_NEW_BROWSER_USING_SECURE_SESSION = 0x04
}
CatLaunchBrowserOptions;

typedef enum
{
  STORE_DATA_IN_TX_BUFFER = 0x00,
  SEND_DATA_IMMEDIATELY = 0x01
}
CatSendDataOptions;

typedef enum
{
  REGISTRY_OK,
  REGISTRY_EVENT_NOT_SUPPORTED,
  REGISTRY_EVENT_ALREADY_REGISTERED,
  REGISTRY_EVENT_NOT_ALLOWED,
  REGISTRY_MENU_STRING_TOO_LONG,
  REGISTRY_BAD_MENU_ENTRY,
  REGISTRY_UNKNOWN_OPERATION,
  REGISTRY_FULL
}
CatRegistryError;

typedef enum
{
  BEARER_SMS = 0x00,
  BEARER_CSD = 0x01,
  BEARER_USSD = 0x02,
  BEARER_GPRS = 0x03
}
CatBearer;

typedef enum
{
  OPEN_CHANNEL_ON_DEMAND = 0x00,
  OPEN_CHANNEL_IMMEDIATELY = 0x01
}
CatOpenChannelOptions;

typedef enum
{
  TRANSPORT_PROTOCOL_UDP = 0x01,
  TRANSPORT_PROTOCOL_TCP = 0x02
}
CatTransportProtocol;

typedef struct
{
  BYTE hour;
  BYTE minute;
  BYTE second;
}
CatTimerValue;


typedef struct
{
  BYTE timeUnit;
  BYTE timeInterval;
}
CatTimeInterval;


typedef struct
{
  BYTE UseIcon;
  BYTE IconIdentifier;
  BYTE IconOptions;
}
CatIconIdentifier;

typedef struct
{
  CatSetupCallOptions Options;

  BYTE UserConfirmationMessageLength;
  const BYTE *UserConfirmationMessage;
  CatIconOption UserConfirmationIcon;

  BYTE CallSetupMessageLength;
  const BYTE *CallSetupMessage;
  CatIconOption CallSetupIcon;

  BYTE AddressLength;
  const BYTE *Address;
  BYTE CalledPartySubaddressLength;
  const BYTE *CalledPartySubaddress;
  BYTE CapabilityConfigParamsLength;
  const BYTE *CapabilityConfigParams;
  CatTimeInterval RedialMaximumDuration;
}
CatSetupCallExParams;

typedef struct
{
  /* Mandatory fields */
  BYTE URLLength;
  const void *URL;
  /* Optional fields */
  BYTE TitleLength;
  const BYTE *Title;

  CatIconIdentifier IconIdentifier;

  BYTE BrowserIdentityLength;
  const void *BrowserIdentity;

  BYTE BearerLength;
  const BYTE *Bearer;

  BYTE GatewayProxyIdLength;
  const void *GatewayProxyId;
}
CatLaunchBrowserExParams;

/*
** General Result Codes
*/
#define GR_OK                                                           0x00
#define GR_PARTIAL                                                      0x01
#define GR_MISSING_INFORMATION                                          0x02
#define GR_REFRESH_WITH_ADDITIONAL_EFS                                  0x03
#define GR_OK_WITHOUT_ICON                                              0x04
#define GR_CALL_CONTROL_MODIFIED                                        0x05
#define GR_OK_LIMITED_SERVICE                                           0x06
#define GR_OK_WITH_MODIFICatION                                         0x07
#define GR_REFRESH_WITHOUT_ACTIVE_NAA                                   0x08
#define GR_USER_TERMINATION                                             0x10
#define GR_QUIT                                                         0x10
#define GR_BACKWARD_MOVE                                                0x11
#define GR_NO_RESPONSE                                                  0x12
#define GR_HELP_REQUEST                                                 0x13
#define GR_SS_TERMINATION                                               0x14
#define GR_RETRY_LOW                                                    0x20
#define GR_TERMINAL_BUSY                                                0x20
#define GR_RETRY_HIGH                                                   0x21
#define GR_USER_REJECTS_COMMAND                                         0x22
#define GR_USER_CLEARED_CALL                                            0x23
#define GR_TIMER_CONFLICT                                               0x24
#define GR_NAA_CALL_CONTROL_CONFLICT                                    0x25
#define GR_BROWSER_LAUNCH_ERROR                                         0x26
#define GR_NO_RETRY_LOW                                                 0x30
#define GR_TERMINAL_UNABLE                                              0x30
#define GR_COMMAND_NOT_RECOGNIZED                                       0x31
#define GR_DATA_NOT_RECOGNIZED                                          0x32
#define GR_COMMAND_NUMBER_NOT_RECOGNIZED                                0x33
#define GR_MISSING_VALUES                                               0x36
#define GR_MULTIPLE_CARDS_ERROR                                         0x38
#define GR_PERMANENT_NAA_PROBLEM                                        0x39
#define GR_BEARER_INDEPENDENT_PROTOCOL_ERROR                            0x3A
#define GR_ACCESS_TECHNOLOGY_INCAPABLE                                  0x3B
#define GR_NO_RETRY_HIGH                                                0x3F
#define GR_DEFAULT                                                      0xFF

/*
** Registry Options
*/
#define REGISTRY_ADD                                                    0x00
#define REGISTRY_DELETE                                                 0x01

/*
** Well-Known FIDs
*/
#define FID_MF			0x3F00
#define FID_DF_GSM		0x7F20
#define FID_DF_GRAPHICS 0x5F50
#define FID_DF_TELECOM  0x7F10
#define FID_EF_ACM	    0x6F39
#define FID_EF_ADN      0x6F3A
#define FID_EF_ARR      0x2F06
#define FID_EF_BDN      0x6F4D
#define FID_EF_CCP      0x6F3D
#define FID_EF_DIR      0x2F00
#define FID_EF_EXT1     0x6F4A
#define FID_EF_EXT2     0x6F4B
#define FID_EF_EXT3     0x6F4C
#define FID_EF_EXT4     0x6F4E
#define FID_EF_FDN      0x6F3B
#define FID_EF_ICCID    0x2FE2
#define FID_EF_IMG      0x4F20
#define FID_EF_LND      0x6F44
#define FID_EF_MSISDN   0x6F40
#define FID_EF_PL       0x2F05
#define FID_EF_SDN      0x6F49
#define FID_EF_SMS      0x6F3C
#define FID_EF_SMSP     0x6F42
#define FID_EF_SMSR     0x6F47
#define FID_EF_SMSS     0x6F43

/*
** File System Interface Functions
*/
WORD
CatSelect(CatFID FileIdentifier, 
          CatFileStatus *Status);

WORD 
CatStatus(CatFileStatus *Status);

void
CatGetCHVStatus (BYTE CHVStatus[4]);

WORD
CatReadBinary (DWORD Offset, 
               DWORD *NumBytes, 
               void *Buffer);


WORD
CatUpdateBinary (DWORD Offset, 
                 DWORD NumBytes, 
                 const void *Buffer);

WORD
CatReadRecord (DWORD RecordNumber, 
               CatRecordAccessMode Mode, 
               DWORD Offset, 
               DWORD *NumBytes, 
               void *Buffer);

WORD
CatUpdateRecord (DWORD RecordNumber, 
                 CatRecordAccessMode Mode, 
                 DWORD Offset, 
                 DWORD NumBytes, 
                 const void *Buffer);

WORD
CatIncrease(DWORD Increment, 
            DWORD *Value);


WORD
CatSearch (CatSearchMode Mode, 
         DWORD Offset, 
         DWORD PatternLength,
         const void *Pattern);

WORD
CatInvalidate (void);


WORD
CatRehabilitate (void);

/*
** High-Level Interface Functions
*/

CatFrameworkEventType
CatGetFrameworkEvent(void);

CatGeneralResult
CatSetupMenu(BYTE TitleLength, 
             const void *Title);

CatGeneralResult
CatSetupEventList();

CatFrameworkEventType CatGetInvocationReason(void);

void CatExit(void);

void CatAddItem(BYTE ItemTextLength, 
                const void *ItemText, 
                BYTE ItemIdentifier);

void CatSelectItem(BYTE TitleLength, const void *Title,
                   CatSelectItemOptions Options);

CatGeneralResult
CatEndSelectItem(BYTE *SelectedItem,
                 const CatIconIdentifier *IconIdentifier);

CatGeneralResult
CatDisplayText(CatDCSValue TextDCS, BYTE TextLength, const void *Text,
               CatDisplayTextOptions Options,
               const CatIconIdentifier *IconIdentifier,
               BYTE ImmediateResponse);

CatGeneralResult
CatGetInKey(CatDCSValue TitleDCS, BYTE TitleLength, const void *Title,
            CatGetInKeyOptions Options, 
            const CatIconIdentifier *IconIdentifier,
            CatDCSValue *DCSOut, void *ByteOut);

CatGeneralResult
CatGetInput(CatDCSValue TitleDCS, BYTE TitleLength, const void *Title, 
            CatGetInputOptions Options, 
            CatDCSValue DefaultReplyDCS,
            BYTE DefaultReplyLength, const void *DefaultReply, 
            BYTE MinimumResponseLength, 
            BYTE MaximumResponseLength, 
            const CatIconIdentifier *IconIdentifier,
            CatDCSValue *MsgOutDCS, 
            void *MsgOutLength,
            void *MsgOut); 

CatGeneralResult
CatSetupIdleModeText(CatDCSValue TextDCS, BYTE TextLength, const void *Text,
                      const CatIconIdentifier *IconIdentifier);

CatGeneralResult
CatPlayTone(BYTE TextLength, const void *Text,
             CatTone Tone,
             CatTimeUnit Units, BYTE Duration,
             const CatIconIdentifier *IconIdentifier);

CatGeneralResult
CatProvideLocalInformation(CatProvideLocalInformationOptions Options,
                            BYTE *LocalInformationLen,
                            void *LocalInformation);

CatGeneralResult CatSetupCall(BYTE UserConfirmationMessageLength, 
              const void *UserConfirmationMessage, 
              CatTypeOfNumberAndNumberingPlanIdentifier TONandNPI, 
              BYTE DiallingNumberLength, const void *DiallingNumber, 
              CatSetupCallOptions Options,
              const CatIconIdentifier *UserConfirmationIconIdentifier,
              BYTE CallSetupMessageLength, const void *CallSetupMessage,
              const CatIconIdentifier *CallSeupIconIdentifier);

CatGeneralResult
CatSetupCallEx(const CatSetupCallExParams *Params);

CatGeneralResult
CatSendShortMessage(BYTE TitleLength, const void *Title, 
           CatTypeOfNumberAndNumberingPlanIdentifier TONandNPI, 
           BYTE AddressLength, const void *Address, 
           BYTE SmsTPDULength, const void *SmsTPDU,
           CatSendShortMessageOptions Options,
           const CatIconIdentifier *IconIdentifier);

CatGeneralResult CatSendSS(BYTE TitleLength, const void *Title, 
           CatTypeOfNumberAndNumberingPlanIdentifier TONandNPI, 
           BYTE SSStringLength, const void *SSString,
           const CatIconIdentifier *IconIdentifier);


CatGeneralResult CatSendUSSD(BYTE TitleLength, const void *Title, 
             CatDCSValue MessageDCS, BYTE MessageLength, const void *Message,
             CatDCSValue *MsgOutDCS, BYTE *MsgOutLength, void *MsgOut,
             const CatIconIdentifier *IconIdentifier);

CatGeneralResult CatOpenCSChannel(CatOpenChannelOptions Options,
                 BYTE UserConfirmationLength, const void *UserConfirmation,
                 const CatIconIdentifier *UserConfimationIconIdentifier,
                 CatTypeOfNumberAndNumberingPlanIdentifier TONandNPI, 
                 BYTE DiallingNumberLength, const void *DiallingNumber,
                 BYTE BearerDescription[3], WORD *BufferSize, CatDevice *ChannelIdentifier);

CatGeneralResult CatOpenGPRSChannel(CatOpenChannelOptions Options,
                 BYTE UserConfirmationLength, const void *UserConfirmation,
                 const CatIconIdentifier *UserConfirmationIconIdentifier, 
                 BYTE BearerDescription[8], WORD *BufferSize, CatDevice *ChannelIdentifier);

BYTE
CatGetTimer(void);

CatGeneralResult
CatFreeTimer(BYTE TimerID);

CatGeneralResult
CatStartTimer(BYTE TimerID, const CatTimerValue *TimerValue);

CatGeneralResult
CatGetTimerValue(BYTE TimerID, CatTimerValue *TimerValue);

CatGeneralResult
CatPowerOnCard(CatDevice DeviceID, BYTE *ATRLength, void *ATR);

CatGeneralResult
CatPowerOffCard(CatDevice DeviceID);

CatGeneralResult
CatPerformCardAPDU(CatDevice DeviceID,
                   BYTE CAPDULength, const void *CAPDU,
                   BYTE *RAPDULength, void *RAPDU);

CatGeneralResult
CatGetReaderStatus(CatDevice DeviceID, 
                   CatGetReaderStatusOptions Options, 
                   BYTE *Status);

CatGeneralResult
CatMoreTime(void);

CatGeneralResult
CatRunATCommand(BYTE TitleLength, const void *Title, 
                BYTE CommandLength, const void *Command,
                const CatIconIdentifier *IconIdentifier,
                BYTE *ResponseLength, void *Response);

CatGeneralResult CatSendDTMFCommand(BYTE TitleLength, const void *Title,
                    BYTE DTMFCodeLength, const void *DTMFCode,
                    const CatIconIdentifier *IconIdentifier);


CatGeneralResult
CatGetLocationInformation (CatLocationInformation *LocationInformation);

CatGeneralResult 
CatGetTimingAdvance(CatTimingAdvance *TimingAdvance);

CatGeneralResult CatGetIMEI(BYTE IMEI[8]);

CatGeneralResult CatGetNetworkMeasurementResults(BYTE MeasurementResults[10]);

CatGeneralResult CatGetDateTimeAndTimeZone(BYTE DateTimeAndTimeZone[7]);

CatGeneralResult CatGetLanguage(BYTE Language[2]);

CatGeneralResult
CatSendDTMF(BYTE TitleLength, const void *Title,
            BYTE DTMFCodeLength, const void *DTMFCode,
            const CatIconIdentifier *IconIdentifier);

void 
CatGetTerminalProfile(BYTE *ProfileOutLength, BYTE *Profile);

CatGeneralResult
CatPollingOff(void);

CatGeneralResult 
CatPollInterval(CatTimeUnit Unit, 
                BYTE Interval,
                CatTimeInterval *ActualIntervalOut);

CatGeneralResult
CatRefresh(CatRefreshOptions Options);

CatGeneralResult
CatRefreshWithFileList(CatRefreshOptions Options,
                       BYTE FileListLength,
                       const void *FileList);

CatGeneralResult
CatLanguageNotification(CatLanguageNotificationOptions Options,
                        const void *Language);

CatGeneralResult
CatLaunchBrowser(CatLaunchBrowserOptions Options,
                 BYTE TitleLength, const void *Title,
                 BYTE URLLength, const void *URL,
                 const CatIconIdentifier *IconIdentifier);

CatGeneralResult
CatLaunchBrowserEx(const CatLaunchBrowserExParams *params);

/*
** Bearer Independent Protocol Functions
*/
CatGeneralResult CatCloseChannel(CatDevice ChannelIdentifier,
                 BYTE TitleLength, const void *Title,
                 const CatIconIdentifier *IconIdentifier);

CatGeneralResult CatReceiveData(CatDevice ChannelIdentifier,
                BYTE TitleLength, const void *Title, BYTE RequestedChannelDataLength,
                const CatIconIdentifier *IconIdentifier,
                BYTE *ChannelData, BYTE *NumChannelBytesRead, BYTE *NumChannelBytesLeft);

CatGeneralResult CatSendData(CatDevice ChannelIdentifier,
             CatSendDataOptions Options, BYTE TitleLength, const void *Title,
             BYTE ChannelDataLength, const void *ChannelData,
             const CatIconIdentifier *IconIdentifier, BYTE *ActualBytesSent);

CatGeneralResult 
CatGetChannelStatus(CatDevice ChannelIdentifier, void *ChannelStatus);

CatGeneralResult
CatServiceSearch(CatBearer BearerId, 
                             BYTE AttributeLength, void *AttributeInformation, 
                             void *ServiceAvailability);

CatGeneralResult CatGetServiceInformation(BYTE TitleLength, const BYTE *Title,
                const CatIconIdentifier *IconIdentifier,
                BYTE BearerId, BYTE *AttributeLength, void *Attributes,
                void *ServiceInformation);

CatGeneralResult CatDeclareService(BYTE BearerId, BYTE ServiceId,
                CatTransportProtocol TransportProtocol,
                WORD *PortNumber, BYTE ServiceRecordLength, void *ServiceRecord);

/*
** Registry Functions
*/
CatRegistryError
CatUpdateRegistry(BYTE AIDLength, const BYTE *AID, const BYTE *Tlv, BYTE Option);

void
CatSetMenuString (BYTE MenuID,
                  BYTE MenuStringLength, const void *MenuString,
                  const CatIconIdentifier *IconIdentifier,
                  BYTE HelpAvailable, 
                  BYTE NextAction);
                  
void
CatNotifyOnFrameworkEvent(CatFrameworkEventType Event, BYTE Enabled);

void
CatNotifyOnEnvelope(CatEnvelopeTagType Tag, BYTE Enabled);

void
CatNotifyOnEvent(CatEventType EventType, BYTE Enabled);


/*
** Low-Level Interface Functions
*/
void
CatResetBuffer( void );

void
CatStartProactiveCommand(BYTE Command,
                         BYTE Options,
                         BYTE To);

CatGeneralResult
CatSendProactiveCommand(BYTE *Length);

CatEnvelopeTagType 
CatOpenEnvelope(BYTE *Length);

void
CatSendEnvelopeResponse (void);

void
CatSendEnvelopeErrorResponse (void);

void
CatSendEnvelopeResponse (void);

void
CatSendEnvelopeErrorResponse (void);

void
CatPutData(BYTE Length, const void *Data);

void
CatPutByte(BYTE Data);

void 
CatPutTLV(BYTE Tag, BYTE Length, const void *Value);

void
CatAppendTLV(BYTE Tag, 
             BYTE Length, 
             const void *Value);

void
CatPutBytePrefixedTLV(BYTE Tag, 
                      BYTE Prefix,
                      BYTE Length, 
                      const void *Value);

void
CatPutOneByteTLV(BYTE Tag, 
                 BYTE Value);

void
CatPutTwoByteTLV(BYTE Tag, 
                 BYTE Value1,
                 BYTE Value2);

BYTE
CatGetByte(void);

const void *
CatGetData(BYTE Length);

const void *
CatFindNthTLV(BYTE Tag, 
              BYTE Occurrence, 
              BYTE *Length);
const void *
CatFindNthTLVInUserBuffer(BYTE BufferLen, 
                      const BYTE *Buffer, 
                      BYTE Tag, 
                      BYTE Occurrence, 
                      BYTE *Length);

/*
** UICC to Terminal Templates
*/
#define PROACTIVE_UICC_COMMAND_TAG                                              0xD0

/*
** Terminal to UICC Templates
*/
#define SMS_PP_DOWNLOAD_TAG                                                     0xD1
#define CELL_BROADCAST_DOWNLOAD_TAG                                             0xD2
#define MENU_SELECTION_TAG                                                      0xD3
#define CALL_CONTROL_TAG                                                        0xD4
#define MO_SHORT_MESSAGE_CONTROL_TAG                                            0xD5
#define EVENT_DOWNLOAD_TAG                                                      0xD6
#define TIMER_EXPIRATION_TAG                                                    0xD7

/*
** Simple TLV Tags
*/
#define COMMAND_DETAILS_TAG                                                     0x01
#define DEVICE_IDENTITY_TAG                                                     0x02
#define RESULT_TAG                                                              0x03
#define DURATION_TAG                                                            0x04
#define ALPHA_IDENTIFIER_TAG                                                    0x05
#define ADDRESS_TAG                                                             0x06
#define CAPABILITY_CONFIGURATION_PARAMETERS_TAG                                 0x07
#define SUBADDRESS_TAG                                                          0x08
#define SS_STRING_TAG                                                           0x09
#define USSD_STRING_TAG                                                         0x0A
#define SMS_TPDU_TAG                                                            0x0B
#define CELL_BROADCAST_PAGE_TAG                                                 0x0C
#define TEXT_STRING_TAG                                                         0x0D
#define TONE_TAG                                                                0x0E
#define ITEM_TAG                                                                0x0F
#define ITEM_IDENTIFIER_TAG                                                     0x10
#define RESPONSE_LENGTH_TAG                                                     0x11
#define FILE_LIST_TAG                                                           0x12
#define LOCATION_INFORMATION_TAG                                                0x13
#define IMEI_TAG                                                                0x14
#define HELP_REQUEST_TAG                                                        0x15
#define NETWORK_MEASUREMENT_RESULTS_TAG                                         0x16
#define DEFAULT_TEXT_TAG                                                        0x17
#define ITEMS_NEXT_ACTION_INDICATOR_TAG                                         0x18
#define EVENT_LIST_TAG                                                          0x19
#define CAUSE_TAG                                                               0x1A
#define LOCATION_STATUS_TAG                                                     0x1B
#define TRANSACTION_IDENTIFIER_TAG                                              0x1C
#define BCCH_CHANNEL_LIST_TAG                                                   0x1D
#define ICON_IDENTIFIER_TAG                                                     0x1E
#define ITEM_ICON_IDENTIFIER_LIST_TAG                                           0x1F
#define CARD_READER_STATUS_TAG                                                  0x20
#define CARD_ATR_TAG                                                            0x21
#define C_APDU_TAG                                                              0x22
#define R_APDU_TAG                                                              0x23
#define TIMER_IDENTIFIER_TAG                                                    0x24
#define TIMER_VALUE_TAG                                                         0x25
#define DATE_TIME_AND_TIME_ZONE_TAG                                             0x26
#define CALL_CONTROL_REQUESTED_ACTION_TAG                                       0x27
#define AT_COMMAND_TAG                                                          0x28
#define AT_RESPONSE_TAG                                                         0x29
#define BC_REPEAT_INDICATOR_TAG                                                 0x2A
#define IMMEDIATE_RESPONSE_TAG                                                  0x2B
#define DTMF_STRING_TAG                                                         0x2C
#define LANGUAGE_TAG                                                            0x2D
#define TIMING_ADVANCE_TAG                                                      0x2E
#define AID_TAG                                                                 0x2F
#define BROWSER_IDENTITY_TAG                                                    0x30
#define URL_TAG                                                                 0x31
#define BEARER_TAG                                                              0x32
#define PROVISIONING_REFERENCE_FILE_TAG                                         0x33
#define BROWSER_TERMINATION_CAUSE_TAG                                           0x34
#define BEARER_DESCRIPTION_TAG                                                  0x35
#define CHANNEL_DATA_TAG                                                        0x36
#define CHANNEL_DATA_LENGTH_TAG                                                 0x37
#define CHANNEL_STATUS_TAG                                                      0x38
#define BUFFER_SIZE_TAG                                                         0x39
#define CARD_READER_IDENTIFIER_TAG                                              0x3A
#define UICC_TERMINAL_INTERFACE_TRANSPORT_LEVEL_TAG                             0x3C
#define DATA_DESTINATION_ADDRESS_TAG                                            0x3E
#define ACCESS_TECHNOLOGY_TAG                                                   0x3F
#define DISPLAY_PARAMETERS_TAG                                                  0x40
#define SERVICE_RECORD_TAG                                                      0x41
#define DEVICE_FILTER_TAG                                                       0x42
#define SERVICE_SEARCH_TAG                                                      0x43
#define ATTRIBUTE_INFORMATION_TAG                                               0x44
#define SERVICE_AVAILABILITY_TAG                                                0x45
#define ESN_TAG                                                                 0x46
#define NETWORK_ACCESS_NAME_TAG                                                 0x47

/*
** Proactive Commands ("Type of Command")
*/
#define REFRESH_COMMAND                                                         0x01
#define MORE_TIME_COMMAND                                                       0x02
#define POLL_INTERVAL_COMMAND                                                   0x03
#define POLLING_OFF_COMMAND                                                     0x04
#define SET_UP_EVENT_LIST_COMMAND                                               0x05
#define SET_UP_CALL_COMMAND                                                     0x10
#define SEND_SS_COMMAND                                                         0x11
#define SEND_USSD_COMMAND                                                       0x12
#define SEND_SHORT_MESSAGE_COMMAND                                              0x13
#define SEND_DTMF_COMMAND                                                       0x14
#define LAUNCH_BROWSER_COMMAND                                                  0x15
#define PLAY_TONE_COMMAND                                                       0x20
#define DISPLAY_TEXT_COMMAND                                                    0x21
#define GET_INKEY_COMMAND                                                       0x22
#define GET_INPUT_COMMAND                                                       0x23
#define SELECT_ITEM_COMMAND                                                     0x24
#define SET_UP_MENU_COMMAND                                                     0x25
#define PROVIDE_LOCAL_INFORMATION_COMMAND                                       0x26
#define TIMER_MANAGEMENT_COMMAND                                                0x27
#define SET_UP_IDLE_MODE_TEXT_COMMAND                                           0x28
#define PERFORM_CARD_APDU_COMMAND                                               0x30
#define POWER_ON_CARD_COMMAND                                                   0x31
#define POWER_OFF_CARD_COMMAND                                                  0x32
#define GET_READER_STATUS_COMMAND                                               0x33
#define RUN_AT_COMMAND_COMMAND                                                  0x34
#define LANGUAGE_NOTIFICATION_COMMAND                                           0x35
#define OPEN_CHANNEL_COMMAND                                                    0x40
#define CLOSE_CHANNEL_COMMAND                                                   0x41
#define RECEIVE_DATA_COMMAND                                                    0x42
#define SEND_DATA_COMMAND                                                       0x43
#define GET_CHANNEL_STATUS_COMMAND                                              0x44
#define SERVICE_SEARCH_COMMAND                                                  0x45
#define GET_SERVICE_INFORMATION_COMMAND                                         0x46
#define DECLARE_SERVICE_COMMAND        

#endif  /* !defined(__CSIMAPI_H_INCLUDED__) */
